@@ -164,6 +164,8 @@ use File::Glob qw(:case);
 	    'lib/autodie/hints.pm',
 	    't/exceptions.t',
 	    't/lib/Hints_pod_examples.pm',
+            # CPAN RT#127762
+            't/recv.t',
         ],
     },
 
@@ -225,7 +227,7 @@ use File::Glob qw(:case);
     },
 
     'Config::Perl::V' => {
-        'DISTRIBUTION' => 'HMBRAND/Config-Perl-V-0.30.tgz',
+        'DISTRIBUTION' => 'HMBRAND/Config-Perl-V-0.31.tgz',
         'FILES'        => q[cpan/Config-Perl-V],
         'EXCLUDED'     => [qw(
 		examples/show-v.pl
@@ -320,7 +322,7 @@ use File::Glob qw(:case);
     },
 
     'Data::Dumper' => {
-        'DISTRIBUTION' => 'SMUELLER/Data-Dumper-2.161.tar.gz',
+        'DISTRIBUTION' => 'XSAWYERX/Data-Dumper-2.173.tar.gz',
         'FILES'        => q[dist/Data-Dumper],
     },
 
@@ -337,7 +339,7 @@ use File::Glob qw(:case);
     },
 
     'Devel::PPPort' => {
-        'DISTRIBUTION' => 'XSAWYERX/Devel-PPPort-3.42.tar.gz',
+        'DISTRIBUTION' => 'XSAWYERX/Devel-PPPort-3.43.tar.gz',
         'FILES'        => q[dist/Devel-PPPort],
         'EXCLUDED'     => [
             'PPPort.pm',    # we use PPPort_pm.PL instead
@@ -418,7 +420,7 @@ use File::Glob qw(:case);
     },
 
     'Exporter' => {
-        'DISTRIBUTION' => 'TODDR/Exporter-5.72.tar.gz',
+        'DISTRIBUTION' => 'TODDR/Exporter-5.73.tar.gz',
         'FILES'        => q[dist/Exporter],
         'EXCLUDED' => [
             qw( t/pod.t
@@ -783,7 +785,7 @@ use File::Glob qw(:case);
     },
 
     'Module::CoreList' => {
-        'DISTRIBUTION' => 'BINGOS/Module-CoreList-5.20180920.tar.gz',
+        'DISTRIBUTION' => 'BINGOS/Module-CoreList-5.20181120.tar.gz',
         'FILES'        => q[dist/Module-CoreList],
     },
 
@@ -1048,7 +1050,7 @@ use File::Glob qw(:case);
     },
 
     'Test::Simple' => {
-        'DISTRIBUTION' => 'EXODIST/Test-Simple-1.302140.tar.gz',
+        'DISTRIBUTION' => 'EXODIST/Test-Simple-1.302141.tar.gz',
         'FILES'        => q[cpan/Test-Simple],
         'EXCLUDED'     => [
             qr{^examples/},
@@ -1138,7 +1140,7 @@ use File::Glob qw(:case);
     },
 
     'threads::shared' => {
-        'DISTRIBUTION' => 'JDHEDDEN/threads-shared-1.58.tar.gz',
+        'DISTRIBUTION' => 'JDHEDDEN/threads-shared-1.59.tar.gz',
         'FILES'        => q[dist/threads-shared],
         'EXCLUDED'     => [
             qw( examples/class.pl
