@@ -9,7 +9,7 @@ BEGIN {
     $ENV{TEST2_ACTIVE} = 1;
 }
 
-our $VERSION = '1.302122';
+our $VERSION = '1.302141';
 
 
 my $INST;
@@ -70,7 +70,7 @@ use Test2::Event::Subtest();
 
 use Carp qw/carp croak confess/;
 use Scalar::Util qw/blessed weaken/;
-use Test2::Util qw/get_tid clone_io pkg_to_file/;
+use Test2::Util qw/get_tid clone_io pkg_to_file gen_uid/;
 
 our @EXPORT_OK = qw{
     context release
@@ -97,6 +97,10 @@ our @EXPORT_OK = qw{
     test2_ipc_wait_disable
     test2_ipc_wait_enabled
 
+    test2_add_uuid_via
+
+    test2_add_callback_testing_done
+
     test2_add_callback_context_aquire
     test2_add_callback_context_acquire
     test2_add_callback_context_init
@@ -114,6 +118,8 @@ our @EXPORT_OK = qw{
 
     test2_ipc
     test2_has_ipc
+    test2_ipc_disable
+    test2_ipc_disabled
     test2_ipc_drivers
     test2_ipc_add_driver
     test2_ipc_polling
@@ -176,6 +182,22 @@ sub test2_no_wait {
     $INST->no_wait;
 }
 
+sub test2_add_callback_testing_done {
+    my $cb = shift;
+
+    test2_add_callback_post_load(sub {
+        my $stack = test2_stack();
+        $stack->top; # Insure we have a hub
+        my ($hub) = Test2::API::test2_stack->all;
+
+        $hub->set_active(1);
+
+        $hub->follow_up($cb);
+    });
+
+    return;
+}
+
 sub test2_add_callback_context_acquire   { $INST->add_context_acquire_callback(@_) }
 sub test2_add_callback_context_aquire    { $INST->add_context_acquire_callback(@_) }
 sub test2_add_callback_context_init      { $INST->add_context_init_callback(@_) }
@@ -191,8 +213,15 @@ sub test2_list_exit_callbacks            { @{$INST->exit_callbacks} }
 sub test2_list_post_load_callbacks       { @{$INST->post_load_callbacks} }
 sub test2_list_pre_subtest_callbacks     { @{$INST->pre_subtest_callbacks} }
 
+sub test2_add_uuid_via {
+    $INST->set_add_uuid_via(@_) if @_;
+    $INST->add_uuid_via();
+}
+
 sub test2_ipc                 { $INST->ipc }
 sub test2_has_ipc             { $INST->has_ipc }
+sub test2_ipc_disable         { $INST->ipc_disable }
+sub test2_ipc_disabled        { $INST->ipc_disabled }
 sub test2_ipc_add_driver      { $INST->add_ipc_driver(@_) }
 sub test2_ipc_drivers         { @{$INST->ipc_drivers} }
 sub test2_ipc_polling         { $INST->ipc_polling }
@@ -229,6 +258,7 @@ sub _contexts_ref                  { $INST->contexts }
 sub _context_acquire_callbacks_ref { $INST->context_acquire_callbacks }
 sub _context_init_callbacks_ref    { $INST->context_init_callbacks }
 sub _context_release_callbacks_ref { $INST->context_release_callbacks }
+sub _add_uuid_via_ref              { \($INST->{Test2::API::Instance::ADD_UUID_VIA()}) }
 
 # Private, for use in Test2::IPC
 sub _set_ipc { $INST->set_ipc(@_) }
@@ -276,7 +306,7 @@ sub no_context(&;$) {
     return;
 };
 
-my $CID = 1;
+my $UUID_VIA = _add_uuid_via_ref();
 sub context {
     # We need to grab these before anything else to ensure they are not
     # changed.
@@ -353,13 +383,18 @@ sub context {
     # hit with how often this needs to be called.
     my $trace = bless(
         {
-            frame    => [$pkg, $file, $line, $sub],
-            pid      => $$,
-            tid      => get_tid(),
-            cid      => 'C' . $CID++,
-            hid      => $hid,
-            nested   => $hub->{nested},
+            frame  => [$pkg, $file, $line, $sub],
+            pid    => $$,
+            tid    => get_tid(),
+            cid    => gen_uid(),
+            hid    => $hid,
+            nested => $hub->{nested},
             buffered => $hub->{buffered},
+
+            $$UUID_VIA ? (
+                huuid => $hub->{uuid},
+                uuid  => ${$UUID_VIA}->('context'),
+            ) : (),
         },
         'Test2::EventFacet::Trace'
     );
@@ -620,7 +655,7 @@ sub run_subtest {
         }
     }
 
-    $hub->finalize($trace->snapshot(hid => $hub->hid, nested => $hub->nested, buffered => $buffered), 1)
+    $hub->finalize($trace->snapshot(huuid => $hub->uuid, hid => $hub->hid, nested => $hub->nested, buffered => $buffered), 1)
         if $ok
         && !$hub->no_ending
         && !$hub->ended;
@@ -628,11 +663,12 @@ sub run_subtest {
     my $pass = $ok && $hub->is_passing;
     my $e = $ctx->build_event(
         'Subtest',
-        pass       => $pass,
-        name       => $name,
-        subtest_id => $hub->id,
-        buffered   => $buffered,
-        subevents  => \@events,
+        pass         => $pass,
+        name         => $name,
+        subtest_id   => $hub->id,
+        subtest_uuid => $hub->uuid,
+        buffered     => $buffered,
+        subevents    => \@events,
     );
 
     my $plan_ok = $hub->check_plan;
@@ -1222,6 +1258,14 @@ Check if Test2 believes it is the END phase.
 This will return the global L<Test2::API::Stack> instance. If this has not
 yet been initialized it will be initialized now.
 
+=item test2_ipc_disable
+
+Disable IPC.
+
+=item $bool = test2_ipc_diabled
+
+Check if IPC is disabled.
+
 =item test2_ipc_wait_enable()
 
 =item test2_ipc_wait_disable()
@@ -1299,6 +1343,22 @@ Add a callback that will be called when Test2 is finished loading. This
 means the callback will be run once, the first time a context is obtained.
 If Test2 has already finished loading then the callback will be run immediately.
 
+=item test2_add_callback_testing_done(sub { ... })
+
+This adds your coderef as a follow-up to the root hub after Test2 is finished loading.
+
+This is essentially a helper to do the following:
+
+    test2_add_callback_post_load(sub {
+        my $stack = test2_stack();
+        $stack->top; # Insure we have a hub
+        my ($hub) = Test2::API::test2_stack->all;
+
+        $hub->set_active(1);
+
+        $hub->follow_up(sub { ... }); # <-- Your coderef here
+    });
+
 =item test2_add_callback_context_acquire(sub { ... })
 
 Add a callback that will be called every time someone tries to acquire a
@@ -1357,6 +1417,22 @@ Returns all the post load callback references.
 
 Returns all the pre-subtest callback references.
 
+=item test2_add_uuid_via(sub { ... })
+
+=item $sub = test2_add_uuid_via()
+
+This allows you to provide a UUID generator. If provided UUIDs will be attached
+to all events, hubs, and contexts. This is useful for storing, tracking, and
+linking these objects.
+
+The sub you provide should always return a unique identifier. Most things will
+expect a proper UUID string, however nothing in Test2::API enforces this.
+
+The sub will receive exactly 1 argument, the type of thing being tagged
+'context', 'hub', or 'event'. In the future additional things may be tagged, in
+which case new strings will be passed in. These are purely informative, you can
+(and usually should) ignore them.
+
 =back
 
 =head2 IPC AND CONCURRENCY
