@@ -6,8 +6,8 @@ afs='false'
 afsroot='/afs'
 alignbytes='8'
 aphostname='/bin/hostname'
-archlib='/usr/local/lib/perl5/5.27/unknown'
-archlibexp='/usr/local/lib/perl5/5.27/unknown'
+archlib='/usr/local/lib/perl5/5.29/unknown'
+archlibexp='/usr/local/lib/perl5/5.29/unknown'
 archname='unknown64'
 asctime_r_proto='0'
 bin='/usr/local/bin'
@@ -42,6 +42,7 @@ d_PRIu64='define'
 d_PRIx64='define'
 d_SCNfldbl='define'
 d__fwalk='undef'
+d_accept4='undef'
 d_access='undef'
 d_accessx='undef'
 d_acosh='undef'
@@ -82,7 +83,6 @@ d_class='undef'
 d_clearenv='undef'
 d_closedir='define'
 d_cmsghdr_s='undef'
-d_const='undef'
 d_copysign='undef'
 d_copysignl='undef'
 d_cplusplus='undef'
@@ -117,6 +117,8 @@ d_double_style_vax='undef'
 d_drand48_r='undef'
 d_drand48proto='undef'
 d_dup2='undef'
+d_dup3='undef'
+d_duplocale='undef'
 d_eaccess='undef'
 d_endgrent='undef'
 d_endgrent_r='undef'
@@ -321,6 +323,7 @@ d_memrchr='undef'
 d_mkdir='undef'
 d_mkdtemp='undef'
 d_mkfifo='undef'
+d_mkostemp='undef'
 d_mkstemp='undef'
 d_mkstemps='undef'
 d_mktime64='undef'
@@ -365,6 +368,7 @@ d_pathconf='undef'
 d_pause='undef'
 d_perl_otherlibdirs='undef'
 d_phostname='undef'
+d_pipe2='undef'
 d_pipe='undef'
 d_poll='undef'
 d_portable='undef'
@@ -496,7 +500,6 @@ d_stdio_ptr_lval_sets_cnt='undef'
 d_stdio_stream_array='undef'
 d_stdiobase='undef'
 d_stdstdio='undef'
-d_strchr='undef'
 d_strcoll='undef'
 d_strerror_l='undef'
 d_strerror_r='undef'
@@ -505,6 +508,7 @@ d_strlcat='undef'
 d_strlcpy='undef'
 d_strnlen='undef'
 d_strtod='undef'
+d_strtod_l='undef'
 d_strtol='define'
 d_strtold='undef'
 d_strtold_l='undef'
@@ -770,8 +774,8 @@ package='perl5'
 perl_static_inline='static'
 phostname='hostname'
 pidtype=int
-privlib='/usr/local/lib/perl5/5.27'
-privlibexp='/usr/local/lib/perl5/5.27'
+privlib='/usr/local/lib/perl5/5.29'
+privlibexp='/usr/local/lib/perl5/5.29'
 procselfexe=''
 prototype='undef'
 ptrsize='8'
@@ -821,11 +825,11 @@ sig_name_init='0'
 sig_num_init='0'
 sig_size='1'
 signal_t=int
-sitearch='/usr/local/lib/perl5/5.27/unknown'
-sitearchexp='/usr/local/lib/perl5/5.27/unknown'
-sitelib='/usr/local/lib/perl5/5.27'
+sitearch='/usr/local/lib/perl5/5.29/unknown'
+sitearchexp='/usr/local/lib/perl5/5.29/unknown'
+sitelib='/usr/local/lib/perl5/5.29'
 sitelib_stem='/usr/local/lib/perl5'
-sitelibexp='/usr/local/lib/perl5/5.27'
+sitelibexp='/usr/local/lib/perl5/5.29'
 sizesize=4
 sizetype='size_t'
 socksizetype='int'
